@@ -150,10 +150,13 @@ module Dlist : sig
       (either by explicit removal or because the maximum size is exceeded) *)
   val set_finaliser_before : ('a node -> unit) -> 'a t -> unit
   val get_finaliser_before : 'a t -> ('a node -> unit)
+  val add_finaliser_before : ('a node -> unit) -> 'a t -> unit
+
 
   (** set a function to be called automatically on a piece of data
       just after it disappears from the list
       (either by explicit removal or because the maximum size is exceeded) *)
   val set_finaliser_after : ('a node -> unit) -> 'a t -> unit
   val get_finaliser_after : 'a t -> ('a node -> unit)
+  val add_finaliser_after : ('a node -> unit) -> 'a t -> unit
 end
