@@ -18,6 +18,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)
 
-
+(*
 val parse_condition :
   Xml.xml -> Ocsigen_extensions.request_info -> bool
+*)
+
+val section : Lwt_log_core.section (** use Lwt_log.Section.set_level in order to debug *)
