@@ -31,7 +31,7 @@ let section = Lwt_log.Section.make "ocsigen:mimetype"
 type 'a assoc_item =
   | Extension of extension * 'a
   | File of filename * 'a
-  | Regexp of Netstring_pcre.regexp * 'a
+  | Regexp of Pcre.regexp * 'a
   | Map of 'a MapString.t
 
 type 'a assoc = {
@@ -42,7 +42,7 @@ type 'a assoc = {
 let find_in_assoc file assoc =
   let filename = Filename.basename file in
   let ext =
-    try String.lowercase (Filename.extension_no_directory file)
+    try String.lowercase_ascii (Filename.extension_no_directory file)
     with Not_found -> ""
   in
   let rec aux = function
@@ -66,7 +66,7 @@ let set_default assoc default = { assoc with assoc_default = default }
 
 let update_ext assoc (ext : extension) v =
   { assoc with assoc_list =
-                 Extension (String.lowercase ext, v) :: assoc.assoc_list}
+                 Extension (String.lowercase_ascii ext, v) :: assoc.assoc_list}
 
 let update_file assoc (file : filename) v =
   { assoc with assoc_list = File (file, v) :: assoc.assoc_list}
@@ -153,9 +153,9 @@ let parse_mime_types ~filename : mime_type assoc =
               close_in in_ch;
               map
             with exn ->
-	            Lwt_log.ign_error ~section ~exn
+                    Lwt_log.ign_error ~section ~exn
                "unable to read the mime.types file";
-	           MapString.empty
+                   MapString.empty
            )];
     assoc_default = default_mime_type;
   }
@@ -165,9 +165,9 @@ let default_mime_assoc () =
   let parsed = ref None in
   match !parsed with
     | None ->
-        let file = Ocsigen_config.get_mimefile () in
-        Lwt_log.ign_info_f ~section "Loading mime types in '%s'" file;
-        let map = parse_mime_types file in
+        let filename = !Ocsigen_config_static.mimefile in
+        Lwt_log.ign_info_f ~section "Loading mime types in '%s'" filename;
+        let map = parse_mime_types ~filename in
         parsed := Some map;
         map
     | Some map -> map
