@@ -16,11 +16,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *)
+
+val section : Lwt_log_core.section (** use Lwt_log.Section.set_level in order to debug *)
+
 (** The requested file does not exists *)
 exception Failed_404
+
 (** The requested file cannot be served: does not exists, not
     enough permissions ... *)
 exception Failed_403
+
 (** The file is a directory which we should not display *)
 exception NotReadableDirectory
 
@@ -43,7 +48,6 @@ type resolved =
   | RFile of string
   | RDir of string
 
-
 (** Finds [filename] in the filesystem, with a possible redirection
     if it is a directory. Takes into account the fact that [filename]
     does not exists, is a symlink or is a directory, and raises
@@ -68,10 +72,3 @@ val resolve :
   ?no_check_for:string ->
   request:Ocsigen_extensions.request ->
   filename:string -> unit -> resolved
-
-
-(** Given the local file [file], with a request originating at url
-    [url], returns a viewable content of [file]. Currently, the [url]
-    parameter is used only if [url] is a directory *)
-val content:
-  request:Ocsigen_extensions.request -> file:resolved -> Ocsigen_http_frame.result Lwt.t
